import fs from 'node:fs/promises';
import path from 'node:path';
import prettier from 'prettier';
// import yaml from 'yaml';
import glob from 'glob-promise';
import { compile } from 'json-schema-to-typescript';
import { camelCase, pascalCase } from 'change-case';
import $RefParser from '@apidevtools/json-schema-ref-parser';
/* ========================================================================== */
export async function generateChecker(schemaPath, outDir) {
    // const file = await fs.readFile(schemaPath, 'utf8');
    // const schema = yaml.parse(file) as JSONSchema7;
    const checkerDir = outDir ?? path.dirname(schemaPath);
    const extension = '.checker.ts';
    const schemaName = path.basename(schemaPath).split('.')[0];
    const checkerFileName = `${schemaName}${extension}`;
    const checkerPath = path.join(checkerDir, checkerFileName);
    console.log(`→ ${checkerPath}`);
    // const refs = await $RefParser.resolve(schemaPath).then((api) => api.paths());
    const schema = await $RefParser.dereference(schemaPath);
    // console.log({ schema });
    const iFace = pascalCase(path.relative(process.cwd(), path
        .basename(schemaPath, path.extname(schemaPath))
        .replace('.schema', '')));
    /* Add a `type` discriminator with interface name to current schema */
    // if (!schema.properties) {
    //   schema.properties = {};
    // }
    // if (!('type' in schema.properties)) {
    //   schema.properties.type = {
    //     const: iFace,
    //   };
    // }
    // if (!Array.isArray(schema.required)) {
    //   schema.required = [];
    // }
    // if (!schema.required.includes('type')) {
    //   schema.required.push('type');
    // }
    const compiled = await compile(schema, iFace, {
        cwd: path.dirname(schemaPath),
        // declareExternallyReferenced: false,
        // unreachableDefinitions: false,
        $refOptions: {
            resolve: { file: false },
        },
        bannerComment: '',
        // IDEA: Might be better to not embed the schema into the validator itself.
        // It's a performance vs. dedup. choice.
        // bannerComment: `${refs
        //   .map(
        //     (r) =>
        //       `// import ${path.basename(
        //         schemaPath,
        //         path.extname(schemaPath),
        //       )} from "${path.relative(process.cwd(), r)}"`,
        //   )
        //   .join('\n')}`,
    }).catch((e) => {
        console.log(e);
    });
    await fs.mkdir(path.dirname(checkerPath), { recursive: true });
    if (compiled) {
        /* ===================== CODE GENERATION ===================== */
        const data = /* ts */ `/* AUTO-GENERATED BY \`@astro-openapi/schema-checkers\`. Do not edit! */

import { itemChecker, type JSONSchema7 } from '@astro-openapi/schema-checkers/dist/validator.js';

const schema = ${JSON.stringify(schema)} as const;
export { schema as ${camelCase(iFace)}Schema, type JSONSchema7 };

${compiled}
export function check${iFace}(item: unknown, replaceDefective = false) {
  return itemChecker<${iFace}>(item, schema as unknown as JSONSchema7, replaceDefective);
}
`;
        /* ===================== CODE GENERATION ===================== */
        await fs.writeFile(checkerPath, prettier.format(data, { parser: 'typescript' }));
    }
}
export async function generateAllCheckers(baseDir, outDir) {
    const pattern = path.join(process.cwd(), `./${baseDir ?? 'src'}/**/*.schema.yaml`);
    const schemas = await glob(pattern);
    if (schemas.length === 0) {
        console.warn(`No schemas found with pattern \`${pattern}\``);
        return;
    }
    await Promise.all(schemas.map(async (p) => {
        await generateChecker(p, outDir).catch((e) => console.log(e));
    })).catch((e) => console.log(e));
}
