import Ajv from 'ajv';
import addFormats from 'ajv-formats';
// import jsf from 'json-schema-faker';
import { 
// set,
// get,
cloneDeep, } from 'lodash-es';
// /* ========================================================================== */
const ajv = new Ajv({
    allErrors: true /* So it doesn't stop at the first found error */,
    strict: false /* Prevents warnings for valid, but relaxed schemas */,
    // NOTE: We might want to make AJV options accessible to user
    // But we'll have to re-compile each time…
    coerceTypes: true,
    removeAdditional: true,
});
addFormats(ajv);
export async function itemChecker(item, schema, replaceDefective = false) {
    const validate = ajv.compile(schema);
    let clone = {};
    let invalidFrontmatter = false;
    if (item &&
        Object.entries(item).every(([key, val]) => typeof key === 'string' && val !== undefined)) {
        clone = cloneDeep(item);
        if (clone.$schema) {
            delete clone.$schema;
        }
        validate(clone);
        if (validate.errors && replaceDefective) {
            /* Generate dummy data which will replace missing or wrong values */
            const jsf = (await import('json-schema-faker')).JSONSchemaFaker;
            const { get, set } = await import('lodash-es');
            jsf.option({ useDefaultValue: true, useExamplesValue: true });
            const dummies = jsf.generate(schema);
            validate.errors?.forEach((e) => {
                let dotPath = '';
                if (e.keyword === 'required' &&
                    typeof e.params.missingProperty === 'string') {
                    dotPath = e.params.missingProperty;
                }
                else {
                    dotPath = e.instancePath.substring(1).replaceAll('/', '.');
                }
                // eslint-disable-next-line @typescript-eslint/no-unnecessary-type-assertion
                const dummy = get(dummies, dotPath);
                set(clone, dotPath, dummy);
            });
        }
    }
    else {
        invalidFrontmatter = true;
        console.warn('Could not validate frontmatter section.');
    }
    return {
        data: clone,
        errors: validate.errors ??
            (invalidFrontmatter ? ['Invalid frontmatter.'] : undefined),
        original: validate.errors ? item : undefined,
        schema,
    };
}
